package com.example.practicerotorcontroller;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;

import android.os.AsyncTask;
import android.os.Bundle;
import android.app.Activity;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.TextView.OnEditorActionListener;

public class RotorControl extends Activity {

	TextView headingTextView;
	Button getHeadingButton;
	EditText headingEditText;
	NetworkTask networkTask;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_rotor_control);
		headingTextView = (TextView) findViewById(R.id.headingTextView);
		getHeadingButton = (Button) findViewById(R.id.headingButton);
		headingEditText = (EditText) findViewById(R.id.headingEditText);
		
		connectToRotor();

		getHeadingButton.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				getHeading();
			}
		});
		
		headingEditText.setOnEditorActionListener(new OnEditorActionListener() {
			
			@Override
			public boolean onEditorAction(TextView v, int actionId,
					KeyEvent event) {
				Log.d("NT","just before goRotate(): ");
				if(actionId == EditorInfo.IME_ACTION_DONE)
					goRotate(v.getText().toString());
				return false;
			}

			private void goRotate(String string) {
				String cmdString = new String("P " + string + " 0\n");
				Log.d("NT","Sending command: "+ cmdString);
				networkTask.sendDataToRotor(cmdString);
			}
		});
	}

	private void connectToRotor() {
		networkTask = new NetworkTask();
		networkTask.execute((Void[])null);
	}

	protected void getHeading() {
		headingTextView.setText("Working....");
		networkTask.sendDataToRotor("p\n");
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.rotor_control, menu);
		return true;
	}

	//192.168.2.153
	public class NetworkTask extends AsyncTask<Void[], byte[], Boolean> {
		

		private Socket nSocket;
		private InputStream nIS;
		private OutputStream nOS;

		@Override
		protected Boolean doInBackground(Void[]... params) {
			Boolean errorConnecting;
			errorConnecting = Boolean.FALSE;
			int nRead;
			SocketAddress sockAddress = new InetSocketAddress("172.20.7.145",4533);
			nSocket = new Socket();
			Log.d("NT","In doInBackground()");
			try {
				nSocket.connect(sockAddress, 10000); //Connect with timeout at 10 seconds.
				if(nSocket.isConnected()) {
					Log.d("NT","Socket Connected!");
					nIS = nSocket.getInputStream();
					nOS = nSocket.getOutputStream();
					byte[] buffer = new byte[256];
					while((nRead = nIS.read(buffer,0,256))!= -1) { //could use read(buffer) instead
						byte[] tempdata = new byte[256];
						System.arraycopy(buffer, 0, tempdata, 0, nRead);
						publishProgress(tempdata);
					}
				}
				else {
					Log.i("NT","Could not connect the socket.");
					errorConnecting = Boolean.TRUE;
				}
			}
			catch (IOException e) {
				Log.i("NT","IOException:  Socket not connected!");
				e.printStackTrace();
			}
			return errorConnecting;
		}
		public void sendDataToRotor(String command) {
			try {
				if(nSocket.isConnected()){
					nOS.write(command.getBytes());
				}
				else {
					Log.i("NT","Could not write data to stream, socket not connected.");
				}
			} 
			catch (IOException e) {
				Log.i("NT","Could not write data to stream, IOException.");
				e.printStackTrace();
			}
		}
		@Override
		protected void onCancelled() {
			cleanUp();
			super.onCancelled();
		}

		private void cleanUp() {
			try {nIS.close();
			nOS.close();
			nSocket.close();
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}

		@Override
		protected void onPostExecute(Boolean result) {
			Log.d("NT","OnPostExecute");
			cleanUp();
			super.onPostExecute(result);
		}

		@Override
		protected void onProgressUpdate(byte[]... values) {
			try {
				Log.d("NT","onProgressUpdate");
				headingTextView.setText(new String(values[0]));
			}
			catch (Exception e){
				e.printStackTrace();
			}
		}

	}

}
